//+------------------------------------------------------------------+
//|                                                         Tree.mqh |
//|                            Copyright  2015, Christopher Barklem |
//|                                            http://www.barklem.es |
//+------------------------------------------------------------------+
#property copyright "Copyright  2015, Christopher Barklem"
#property link      "http://www.barklem.es"
#property version   "1.00"
#property strict

#include "TreeNode.mqh"
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
class CTree : public CTreeNode
  {
private:

public:
                     CTree();
                    ~CTree();
   void              SampleTree();

   CTreeNode        *m_TreeRoot;
   CTreeNode        *m_TreeNodes[];
   int               m_TreeNode;

   CTreeNode        *m_TempParent;

protected:

   CTreeNode        *CreateNode(CTreeNode *Parent,double LowPrice,double HighPrice);
   CTreeNode        *InsertNode(CTreeNode *Parent,double LowPrice,double HighPrice);
   bool              DoOverlap(double LowPrice1,double HighPrice1,double LowPrice2,double HighPrice2);
   CTreeNode        *IntervalSearch(CTreeNode *Parent,CTreeNode *Subject);
   void              InOrder(CTreeNode *Node);

  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CTree::CTree()
  {
   m_TreeNode=-1;
   SampleTree();
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CTree::~CTree()
  {
   for(int i=0; i<m_TreeNode; i++) { delete m_TreeNodes[i];}
   delete m_TreeRoot;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CTree::SampleTree()
  {
   double PriceHighLow[5][2];

   PriceHighLow[0][0]=1.256; // this
   PriceHighLow[0][1]=1.270;
   PriceHighLow[1][0]=1.265; // overide this
   PriceHighLow[1][1] = 1.280;
   PriceHighLow[2][0] = 1.315;
   PriceHighLow[2][1] = 1.340;
   PriceHighLow[3][0] = 1.335;
   PriceHighLow[3][1] = 1.500;
   PriceHighLow[4][0] = 1.345;
   PriceHighLow[4][1] = 1.440;

   m_TempParent=(CTreeNode*)NULL;
   for(int i=0; i<ArrayRange(PriceHighLow,0); i++)
     {
      m_TempParent=InsertNode(m_TempParent,PriceHighLow[i][0],PriceHighLow[i][1]);
     }
   Print("Inserted:",m_TreeNode," nodes.");
   for(int i=0; i<m_TreeNode; i++)
     {
      IntervalSearch(m_TreeRoot,m_TreeNodes[i]);
     }
  }
// A utility function to create a new Interval Search Tree Node
CTreeNode *CTree::CreateNode(CTreeNode *Parent,double LowPrice,double HighPrice)
  {
   bool Root=false;
   if(m_TreeNode==-1)
     {
      ArrayResize(m_TreeNodes,1);
      m_TreeNode=1;
      Root=true;
     }
   else
     {
      ArrayResize(m_TreeNodes,m_TreeNode+1);
      m_TreeNode++;
     }
   m_TreeNodes[m_TreeNode-1]=(CTreeNode*)NULL;
   CTreeNode *newCTreeNode=new CTreeNode(Parent,LowPrice,HighPrice);
   m_TreeNodes[m_TreeNode-1]=newCTreeNode;

   if(Root) m_TreeRoot=newCTreeNode;

   return newCTreeNode;

  };
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
CTreeNode *CTree::InsertNode(CTreeNode *Parent,double LowPrice,double HighPrice)
  {

   CTreeNode *Node=(CTreeNode*)NULL;
   if(Parent == NULL) return CreateNode(Node,LowPrice, HighPrice);

   double ParentHighPrice  = Parent.GetHighPrice();
   double ParentLowPrice   = Parent.GetLowPrice();
   double ParentMax        = Parent.GetMaxPrice();
   CTreeNode *LeftNode     = Parent.GetLeftNode();
   CTreeNode *RightNode    = Parent.GetRightNode();

   if(LowPrice<ParentLowPrice)
     {

      // Go Left
      if(LeftNode==NULL)
        {
         if(ParentMax<HighPrice) {Parent.SetMax(HighPrice);}
         Parent.SetLeftNode(CreateNode(Parent,LowPrice,HighPrice));
         return m_TreeRoot;
        }
      else {InsertNode(LeftNode,LowPrice,HighPrice);}
     }
   else
     {
      // Go Right
      if(RightNode==NULL)
        {
         if(ParentMax<HighPrice) {Parent.SetMax(HighPrice);}
         Parent.SetRightNode(CreateNode(Parent,LowPrice,HighPrice));
         return m_TreeRoot;
        }
      else {InsertNode(RightNode,LowPrice,HighPrice);}
     }
   return m_TreeRoot;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
bool CTree::DoOverlap(double LowPrice1,double HighPrice1,double LowPrice2,double HighPrice2)
  {
   if(LowPrice1 <= HighPrice2 && LowPrice2 <= HighPrice1) return true;
   return false;
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
// The main function that searches a given interval i in a given
// Interval Tree.
CTreeNode *CTree::IntervalSearch(CTreeNode *Parent,CTreeNode *Subject)
  {
// Base Case, tree is empty
   if(Parent == NULL) return Parent;

// If given Subject overlaps with Parent

   double ParentHighPrice  = Parent.GetHighPrice();
   double ParentLowPrice   = Parent.GetLowPrice();
   double ParentMax        = Parent.GetMaxPrice();
   CTreeNode *ParentLeftNode     = Parent.GetLeftNode();
   CTreeNode *ParentRightNode    = Parent.GetRightNode();

   double SubjectHighPrice  = Subject.GetHighPrice();
   double SubjectLowPrice   = Subject.GetLowPrice();
   double SubjectMax        = Subject.GetMaxPrice();
//CTreeNode *LeftNode      = Subject.GetLeftNode();
//CTreeNode *RightNode     = Subject.GetRightNode();   

   if(DoOverlap(ParentLowPrice,ParentHighPrice,SubjectLowPrice,SubjectHighPrice))
     {
      Print("ParentLowPrice:",ParentLowPrice," ParentHighPrice:",ParentHighPrice," Overlapds the Node with prices - SubjectLowPrice:",SubjectLowPrice," SubjectHighPrice:",SubjectHighPrice);
      return Parent;
     }
// If left child of Parent is present and max of left child is
// greater than or equal to given interval, then i may
// overlap with an interval is left subtree
   if(ParentLeftNode!=NULL && ParentLeftNode.GetMaxPrice()>=SubjectLowPrice)
      return IntervalSearch(ParentLeftNode, Subject);

// Else interval can only overlap with right subtree
   return IntervalSearch(ParentRightNode, Subject);
  }
//+------------------------------------------------------------------+
//|                                                                  |
//+------------------------------------------------------------------+
void CTree::InOrder(CTreeNode *Node)
  {
   if(Node==NULL)
      return;

   InOrder(Node.GetLeftNode());
   Print("[",Node.GetLowPrice(),", ",Node.GetMaxPrice(),"] max = ",Node.GetMaxPrice());
   InOrder(Node.GetRightNode());
  }
//+------------------------------------------------------------------+
